@@ -3,9 +3,10 @@
 from os import path, kill
 import subprocess, signal
 import sys
+import re
+import psutil
 
 import warnings
-import cjson
 from copy import copy, deepcopy
 from flask import session, g
 from tiramisu.option import *
@@ -21,6 +22,10 @@ from creole.var_loader import modes, modes_level, convert_value, type_option, \
 from creole.config import configeol
 from creole import eosfunc
 
+
+if sys.version_info[0] >= 3:
+    xrange = range
+
 try:
     from collections import OrderedDict
     from os import umask
@@ -30,9 +35,18 @@ except:
     from pyeole.odict import OrderedDict
 
 try:
-    from zephir.lib_zephir import xmlrpclib
+    from zephir.lib_zephir.xmlrpclib import ProtocolError
 except:
-    import xmlrpclib
+    try:
+        from xmlrpclib import ProtocolError
+    except:
+        # python3
+        from xmlrpc.client import ProtocolError
+try:
+    # python3
+    from importlib import reload
+except:
+    pass
 
 #from pyeole.log import init_logging
 #
@@ -44,6 +58,7 @@ warnings.simplefilter("always", ValueWarning)
 
 
 MSG_ERR_IMPORT_VAR = "Problème de chargement de la variable \"{1}\"({2}) de la catégorie \"{0}\""
+PROCESS_RE = re.compile(r'\s*(?P<pid>[0-9]+) .*?((/bin/)?bash( .*)? )?(/usr/sbin/)?gen_config.*', re.M)
 
 class ConfigEmptyError(Exception):
     pass
@@ -70,15 +85,16 @@ def quit_genconfig(id_):
     :param id_: session id
     :type  id_: `str`
     """
-    p = subprocess.Popen(['ps', '-U', 'genconfig', '--no-header'], stdout=subprocess.PIPE)
-    out, err = p.communicate()
-    for line in out.splitlines():
-        if 'uzbl-core' in line:
-            pid = int(line.split(None, 1)[0])
-            kill(pid, signal.SIGKILL)
-        elif 'chromium' in line:
-            pid = int(line.split(None, 1)[0])
-            kill(pid, signal.SIGKILL)
+    p = subprocess.run(['ps', 'af', '--no-headers', '-o', 'pid,cmd'], capture_output=True)
+    try:
+        process_line = PROCESS_RE.search(p.stdout.decode('utf-8'))
+        parent_pid = int(process_line.group('pid'))
+        children_process = psutil.Process(parent_pid).children()
+        for child_process in children_process:
+            child_process.send_signal(signal.SIGTERM)
+    except Exception as err:
+        # Arrêt propre de genconfig échoué
+        pass
 
 def del_config(id_):
     """ Removes the user's current configuration
@@ -102,7 +118,7 @@ def init_session(id_):
     :param id_: session id
     :type  id_: `str`
     """
-    if not session_tiramisu.has_key(id_):
+    if not id_ in session_tiramisu:
         session_tiramisu[id_] = {'config':None, 'mode': 'normal', 'debug': False, 'zephir': { 'available': False, 'reauth': True }}
 
 def init_zephir_session(id_, zephir):
@@ -214,10 +230,10 @@ def get_zephir_config(id_):
         code, res = proxy.serveurs.get_status(zephir['server_id'])
         code, info_serv = proxy.serveurs.get_serveur(zephir['server_id'])
 
-    except xmlrpclib.ProtocolError:
-        print "Accès au serveur %s non autorisé" % zephir['server_id']
-    except Exception, e:
-        print "Erreur de lecture des informations sur Zéphir \n(%s)" % str(e)
+    except ProtocolError:
+        print("Accès au serveur %s non autorisé" % zephir['server_id'])
+    except Exception as e:
+        print("Erreur de lecture des informations sur Zéphir \n(%s)" % str(e))
     else:
         if res['config_ok'] == 0:
             raise ConfigEmptyError("La configuration distante (Zéphir) n'existe pas")
@@ -303,8 +319,6 @@ def get_load_errors(id_):
             try:
                 varname = config.unwrap_from_path('.'.join(path), force_permissive=True).impl_getdoc()
                 varid = path[-1]
-                if isinstance(category_name, unicode):
-                    category_name = unicode.encode(category_name, 'utf8')
                 load_errors.append(MSG_ERR_IMPORT_VAR.format(category_name, varname, varid))
             except PropertiesOptionError:
                 pass
@@ -341,6 +355,8 @@ def get_categories(id_, get_nested=False, init=False, zephir_sync=False):
     else:
         config = get_config(id_)
     ret = []
+    if get_nested:
+        config.cfgimpl_get_settings().remove('cache')
     for name, ownconfig in config.creole.iter_groups():
         optdesc = ownconfig.cfgimpl_get_description()
         help_ = optdesc.impl_get_information('help', '')
@@ -351,7 +367,7 @@ def get_categories(id_, get_nested=False, init=False, zephir_sync=False):
                     'icon': icon,
                     'help': help_}
         has_value = False
-        for var in ownconfig.iter_all():
+        for var in list(ownconfig.iter_all()):
             path = get_path(name, var[0])
             option = config.unwrap_from_path(path)
             type_ = type_option[type(option)]
@@ -376,6 +392,8 @@ def get_categories(id_, get_nested=False, init=False, zephir_sync=False):
                     category['mode'] = mode
                     break
             ret.append(category)
+    if get_nested:
+        config.cfgimpl_get_settings().append('cache')
     return ret
 
 
@@ -465,7 +483,7 @@ def get_variable(id_, category_name, name, slave=None, slaves=None, current_tag=
             else:
                 is_default_owner = config.cfgimpl_get_values().is_default_owner(option, force_permissive=True)
                 is_calculated =  option.impl_has_callback() and is_default_owner
-        except ValueError, err:
+        except ValueError as err:
             is_default_owner = True
             is_calculated = False
             error = err
@@ -1031,11 +1049,15 @@ def diff_config(id_, zephir_sync=False):
                             new_value = []
                             new_owner = []
                             if master in orig_values:
-                                len_master_value = len(orig_values[master]['val'])
+                                master_value = orig_values[master]['val']
                             else:
-                                len_master_value = len(dico['val'])
+                                master_value = dico['val']
+                            if not isinstance(master_value, list):
+                                len_master_value = 0
+                            else:
+                                len_master_value = len(master_value)
                             for idx in xrange(len_master_value):
-                                if str(idx) in dico['val']:
+                                if isinstance(dico['val'], dict) and str(idx) in dico['val']:
                                     new_value.append(dico['val'][str(idx)])
                                     new_owner.append(dico['owner'][str(idx)])
                                 else:
@@ -1113,12 +1135,12 @@ def diff_config(id_, zephir_sync=False):
                 path = config.creole.find_first(byname=key, type_='path',
                                                 force_permissive=True)
                 diffs.append(dict(
-                    get_variable_from_path(id_, path,
-                                           force_permissive=True,
-                                           from_diff=True).items() + {
+                    list(get_variable_from_path(id_, path,
+                                                force_permissive=True,
+                                                from_diff=True).items()) + list({
                         'old_value': store[key]['val'],
                         'old_user': store[key]['owner']
-                    }.items()
+                    }.items())
                 ))
             except AttributeError:
                 #remove older/disabled variable's
@@ -1128,12 +1150,12 @@ def diff_config(id_, zephir_sync=False):
                 path = config.creole.find_first(byname=key, type_='path',
                                                 force_permissive=True)
                 diffs.append(dict(
-                    get_variable_from_path(id_, path,
-                                           force_permissive=True,
-                                           from_diff=True).items() + {
+                    list(get_variable_from_path(id_, path,
+                                                force_permissive=True,
+                                                from_diff=True).items()) + list({
                         'new_value': new_store[key]['val'],
                         'new_user': new_store[key]['owner']
-                    }.items()
+                    }.items())
                 ))
             except AttributeError:
                 #for example when imported config file
@@ -1149,14 +1171,14 @@ def diff_config(id_, zephir_sync=False):
                     path = config.creole.find_first(byname=key, type_='path',
                                                     force_permissive=True)
                     diffs.append(dict(
-                        get_variable_from_path(id_, path,
-                                               force_permissive=True,
-                                               from_diff=True).items() + {
+                        list(get_variable_from_path(id_, path,
+                                                    force_permissive=True,
+                                                    from_diff=True).items()) + list({
                             'old_value': store[key]['val'],
                             'old_user': store[key]['owner'],
                             'new_value': new_store[key]['val'],
                             'new_user': new_store[key]['owner']
-                        }.items()
+                        }.items())
                     ))
                 except AttributeError:
                     pass
@@ -1212,8 +1234,8 @@ def save_zephir_config(id_):
     zephir = get_zephir(id_)
     values = get_values(id_)
     try:
-        code, msg = zephir['proxy'].serveurs.save_conf(zephir['server_id'], [unicode(str(values))])
-    except Exception, e:
+        code, msg = zephir['proxy'].serveurs.save_conf(zephir['server_id'], [str(values)])
+    except Exception as e:
         # FIXME: Log Error messages
         code = 0
         msg = str(e)
